//===========================================================================
// Respell Rhythms
//
// This software is largely derived from the New Retrograde plugin and I
// consider it to fall under
// Copyright (C) 2020 Astrid Lydia Johannsen (ellejohara)
// 
// I make no further claims
//===========================================================================

import QtQuick 2.0
import MuseScore 3.0

MuseScore {
	menuPath: "Plugins.RespellRhythm"
	description: "Copies a selection of notes to a score with prepared (arbitrary) rhythms."
	version: "1.0"
	
	function rerhythm() {
		var tpc2note = ["cbb","gbb","dbb","abb","ebb","bbb","fb","cb","gb","db","ab","eb","bb","f","c","g","d","a","e","b","f#","c#","g#","d#","a#","e#","b#","f##","c##","g##","d##","a##","e##","b##"];									
		var cursor = curScore.newCursor(),
                startstaff,
                endstaff,
                endTick,
				fullScore = false;
		
		cursor.rewind(1); // start of selection
		if (!cursor.segment) { // no selection
			console.log('nothing selected'); // say "nothing selected"
			Qt.quit(); // then quit
        } else {
            var startstaff = cursor.staffIdx;
            cursor.rewind(2);
            if (cursor.tick === 0) {
				console.log ("in cursor.tick 0")
                // this happens when the selection includes
                // the last measure of the score.
                // rewind(2) goes behind the last segment (where
                // there's none) and sets tick=0
                endTick = curScore.lastSegment.tick + 1;
				endstaff = curScore.nstaves - 1; // and end with last
            } else {
                endTick = cursor.tick;
				endstaff = cursor.staffIdx;
            }
        }
		if (endstaff > startstaff){
			console.log('Please select one staff only'); 
			Qt.quit(); // then quit
		}
		var selectionArray = []; // create a blank array
		cursor.rewind(1); // beginning of selection
				
		while(cursor.segment && cursor.tick < endTick) { // while in the selection
			var e = cursor.element; // put current element into variable e
			if(e) { // if e exists
				if(e.type == Element.CHORD) { // if e is a note or chord
					var e = cursor.element; // put current element into variable e
					var pitches = []; // put the note pitches of each chord into an array
					var tones = []; // put tpc values into an array
					var notes = e.notes; // get all the notes of the chord
					for(var i = 0; i < notes.length; i++) { // iterate through each note in chord
						var note = notes[i]; // get the note pitch number
						pitches.push(note.pitch); // push pitch number into array
						tones.push(note.tpc); // push tpc value into array
					}
				}
				if(e.type == Element.REST) { // if e is a rest
					var pitches = 'REST'; // "REST" as pitch
				}
				
				var numer = e.duration.numerator; // numerator of duration
				var denom = e.duration.denominator; // denominator of duration
				selectionArray.push([pitches, tones, numer, denom]);
			} else {
				console.log('nothing happened');
				Qt.quit();
			}
			cursor.next(); // move to next tick
		} // end while
		
		curScore.startCmd();
	    cmd("next-score");
    	

		var cursor = curScore.newCursor();
		if (typeof curScore === 'undefined') {
			console.log ("All done");
			Qt.quit();
		}
		cursor.rewind(0); // beginning of score
		cursor.staffIdx = startstaff;
		curScore.endCmd();

		curScore.startCmd();
		// this section rewrites the selection with the new rhythms
		for(var i = 0; i < selectionArray.length; i++) {
			var selection = selectionArray[i];
			var pitches = selection[0]; // get note and chord pitches
			var tones = selection[1]; // get tpc values
			
			// find the desired duration
			var e = cursor.element; // put current element into variable e
			if(e && e.type == Element.REST) { // if e exists
				var numer = e.duration.numerator; // numerator of duration
				var denom = e.duration.denominator; // denominator of duration
			 }
			 else{
				 console.log ("Destination file does not have (enough) rests")
				 Qt.quit();
			 }
			
			// set the duration
			cursor.setDuration(numer, denom); // set duration of note, chord, rest
			
			if(pitches === 'REST') {
				cursor.addRest () // add rest and advance cursor
			} else { // if note or chord
				for(var j = 0; j < pitches.length; j++) {
						cursor.addNote(pitches[0]); // write note with pitch
						cursor.prev(); // rewind cursor one element
						var note = cursor.element.notes[0];
						note.pitch = pitches[0]; // fix pitch
						note.tpc = tones[0]; // set tpc
						//console.log("note = ", tpc2note[note.tpc])
						if(pitches.length > 1) { // if multiple pitches (i.e. a chord)
							for(var k = 1; k < pitches.length; k++) { // loop through remaining pitches
								var chordNote = newElement(Element.NOTE); // create a new note object
								chordNote.pitch = pitches[k]; // define the pitch
								chordNote.tpc = tones[k]; // define the tpc (it will be ignored)
								//console.log("note = ", tpc2note[note.tpc])
								var curChord = cursor.element; // get the existing note at the current cursor position
								curChord.add(chordNote); // add the new note to the chord root note
								curChord.notes[k].tpc = tones[k]; // set the tpc of the new chord note
							} //end of var k
						} // end of chord pitches
				} // end for var j note or chord
                cursor.next(); // move to the next element
			} // end of if pitches
		} // end of for i = 0		
		curScore.endCmd();
	}
	
	// do the thing
	onRun: {
		curScore.startCmd();
	    rerhythm();
    	curScore.endCmd();
		Qt.quit()
	}
}